@@ -82,7 +82,7 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 
 #ifdef USE_SOCKS
 
-#define CONNECT(A,B,C)  (socks_flag) ? Rconnect((A),(B),(B)) : \
+#define CONNECT(A,B,C)  (socks_flag) ? Rconnect((A),(B),(C)) : \
                                         connect((A),(B),(C))
 #define GETSOCKNAME(A,B,C) (socks_flag) ? Rgetsockname((A),(B),(C)) : \
                                         getsockname((A),(B),(C))
@@ -92,14 +92,16 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 #define ACCEPT(A,B,C)   (socks_flag) ? Raccept((A),(B),(C)) : \
                                         accept((A),(B),(C))
 #if defined(SHORTENED_RBIND)
-#define bind            Rbind
+#define BIND(A,B,C)     (socks_flag) ? Rbind(A,B,C) : \
+                                        bind((A),(B),(C))
 #else /* !SHORTENED_RBIND */
 /*
  * Be sure to put in socks_bind_remoteAddr (32-bit integer) the IP
  * address (in network byte order) of the host it is aiming for,
  * which is typically the destination host of the previous connect().
  */
-#define bind(a,b,c)     Rbind(a,b,c,socks_bind_remoteAddr)
+#define BIND(A,B,C)     (socks_flag) ? Rbind(A,B,C,socks_bind_remoteAddr) : \
+                                        bind((A),(B),(C))
 #endif /* SHORTENED_RBIND */
 #else /* !USE_SOCKS */
 #define CONNECT        connect
@@ -278,7 +280,7 @@ login(host)
 		else
 			user = tmp;
 	}
-	n = auth_user(user,(char *)NULL);
+	n = auth_user(user,pass);
 	if (n == CONTINUE) {
 		aflag++;
 		acct = getpass("Account:");
@@ -989,7 +991,7 @@ recvrequest(cmd, local, remote, lmode, printnames)
 			static char errbuf[1024];
 
 			sprintf(errbuf,"ftp: SSL_read DATA error %s\n",
-				    ERR_error_string(ERR_get_error()));
+				    ERR_error_string(ERR_get_error(),NULL));
 
 			/* tell the user ... who else */
 			fprintf(stderr,"%s", errbuf);
@@ -1167,11 +1169,22 @@ initconn()
 	register char *p, *a;
 	int result, len, tmpno = 0;
 	int on = 1;
+#ifdef USE_SOCKS
+	static int scanport = 5000;
+#endif /* USE_SOCKS */
 
 noport:
 	data_addr = myctladdr;
-	if (sendport)
-		data_addr.sin_port = 0;	/* let system pick one */ 
+	if (sendport) {
+#ifdef USE_SOCKS
+		if (socks_flag) {
+		    data_addr.sin_port = scanport++;	
+		    if (scanport >= 6000)
+			scanport = 5000;
+		} else 
+#endif /* USE_SOCKS */
+		    data_addr.sin_port = 0;	/* let system pick one */ 
+	}
 	if (data != -1)
 		(void) close(data);
 	data = socket(AF_INET, SOCK_STREAM, 0);
@@ -1186,7 +1199,7 @@ noport:
 			perror("ftp: setsockopt (reuse address)");
 			goto bad;
 		}
-	if (bind(data, (struct sockaddr *)&data_addr, sizeof (data_addr)) < 0) {
+	if (BIND(data, (struct sockaddr *)&data_addr, sizeof (data_addr)) < 0) {
 		perror("ftp: bind");
 		goto bad;
 	}
@@ -1289,7 +1302,7 @@ dataconn(lmode)
                 static char errbuf[1024];
 
 	        sprintf(errbuf,"ftp: SSL_connect DATA error %s\n",
-			    ERR_error_string(ERR_get_error()));
+			    ERR_error_string(ERR_get_error(),NULL));
 
 		/* tell the user ... who else */
 		fprintf(stderr,"%s", errbuf);
@@ -1712,62 +1725,64 @@ char *u,*p;
 #endif /* USE_SRA */
 
 #ifdef USE_SSL
-	n = command("AUTH SSL");
-	if (n == ERROR) {		/* do normal USER/PASS */
-	    printf("SSL not available\n");
-	    /* spit the dummy as we will only talk ssl
-	     * when running in "secure" mode 
-	     */
-	    if (ssl_secure_flag)
-		return ERROR;
-	} else if (n == CONTINUE ) {
-	    /* do SSL */
-	    ssl_con=(SSL *)SSL_new(ssl_ctx);
+        if (ssl_enabled) {
+	    n = command("AUTH SSL");
+	    if (n == ERROR) {		/* do normal USER/PASS */
+		printf("SSL not available\n");
+		/* spit the dummy as we will only talk ssl
+		 * when running in "secure" mode 
+		 */
+		if (ssl_secure_flag)
+		    return ERROR;
+	    } else if (n == CONTINUE ) {
+		/* do SSL */
+		ssl_con=(SSL *)SSL_new(ssl_ctx);
 
-	    SSL_set_fd(ssl_con,fileno(cout));
-	    SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
-	    SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
+		SSL_set_fd(ssl_con,fileno(cout));
+		SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
+		SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
 
-	    /* Add in any certificates if you want to here ... */
-	    if (my_ssl_cert_file) {
-		if (!SSL_use_certificate_file(ssl_con, my_ssl_cert_file,
-		    X509_FILETYPE_PEM)) {
-		    fprintf(stderr,"%s: ",my_ssl_cert_file);
-		    ERR_print_errors(stderr);
-		    exit(1);
-		} else {
-		    if (!my_ssl_key_file)
-			my_ssl_key_file = my_ssl_cert_file;
-		    if (!SSL_use_RSAPrivateKey_file(ssl_con, my_ssl_key_file,
+		/* Add in any certificates if you want to here ... */
+		if (my_ssl_cert_file) {
+		    if (!SSL_use_certificate_file(ssl_con, my_ssl_cert_file,
 			X509_FILETYPE_PEM)) {
-			fprintf(stderr,"%s: ", my_ssl_key_file);
+			fprintf(stderr,"%s: ",my_ssl_cert_file);
 			ERR_print_errors(stderr);
 			exit(1);
+		    } else {
+			if (!my_ssl_key_file)
+			    my_ssl_key_file = my_ssl_cert_file;
+			if (!SSL_use_RSAPrivateKey_file(ssl_con, my_ssl_key_file,
+			    X509_FILETYPE_PEM)) {
+			    fprintf(stderr,"%s: ", my_ssl_key_file);
+			    ERR_print_errors(stderr);
+			    exit(1);
+			}
 		    }
 		}
-	    }
 
-	    if (SSL_connect(ssl_con)<=0) {
-                static char errbuf[1024];
+		if (SSL_connect(ssl_con)<=0) {
+		    static char errbuf[1024];
 
-	        sprintf(errbuf,"ftp: SSL_connect error %s\n",
-			    ERR_error_string(ERR_get_error()));
-		perror(errbuf);
-		/* abort time methinks ... */
-		return ERROR;
-	    } else {
-		fprintf(stderr,"[SSL Cipher %s]\n",SSL_get_cipher(ssl_con));
-		fflush(stderr);
-		ssl_active_flag=1;
-	    }
+		    sprintf(errbuf,"ftp: SSL_connect error %s\n",
+				ERR_error_string(ERR_get_error(),NULL));
+		    perror(errbuf);
+		    /* abort time methinks ... */
+		    return ERROR;
+		} else {
+		    fprintf(stderr,"[SSL Cipher %s]\n",SSL_get_cipher(ssl_con));
+		    fflush(stderr);
+		    ssl_active_flag=1;
+		}
 
-	    n = command("USER %s",u);
-	    if (n == CONTINUE) {
-		    if(p == NULL)
-			    p = getpass("Password:");
-		    n = command("PASS %s",p);
+		n = command("USER %s",u);
+		if (n == CONTINUE) {
+			if(p == NULL)
+				p = getpass("Password:");
+			n = command("PASS %s",p);
+		}
+		return (n);
 	    }
-	    return (n);
 	}
 #endif /* USE_SSL */
 #ifdef USE_SRA
