@@ -356,6 +356,9 @@ main(argc, argv, envp)
 		        if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
 			    my_ssl_key_file=optarg+strlen("key=");
 			}
+			/* we have swallowed an extra arg */
+			argc--;
+			argv++;
 			break;
 #endif /* USE_SSL */
 
@@ -445,7 +448,7 @@ nextopt:
 	    ssl_public_cert=X509_new();
 	    if (PEM_read_X509(fp,&ssl_public_cert,NULL)==NULL) {
 		fprintf(stderr,"ftpd: error reading public cert - %s\n",
-                                ERR_error_string(ERR_get_error()));
+                                ERR_error_string(ERR_get_error(),NULL));
 		fflush(stderr);
 		exit(1);
 	    }
@@ -475,7 +478,7 @@ nextopt:
 	    ssl_private_key=RSA_new();
 	    if (PEM_read_RSAPrivateKey(fp,&ssl_private_key,NULL)==0) {
 		fprintf(stderr,"ftpd: error reading private key - %s\n",
-			ERR_error_string(ERR_get_error()));
+			ERR_error_string(ERR_get_error(),NULL));
 		fflush(stderr);
 		exit(1);
 	    }
@@ -1089,6 +1092,11 @@ dataconn(name, size, mode)
 	    }
 	    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
 
+            /* copy session details ... */
+	    SSL_copy_session_id(ssl_data_con,ssl_con);
+
+	    /* for 0.5.2 - want to change the timeout value etc ... */
+
 	    SSL_set_fd(ssl_data_con,data);
 	    SSL_set_verify(ssl_data_con,ssl_verify_flag,NULL);
 
@@ -1104,7 +1112,7 @@ dataconn(name, size, mode)
                 static char errbuf[1024];
 
 	        sprintf(errbuf,"ftpd: SSL_accept DATA error %s\n",
-			    ERR_error_string(ERR_get_error()));
+			    ERR_error_string(ERR_get_error(),NULL));
 		perror_reply(425, errbuf);
 		/* abort time methinks ... */
 		fclose(file);
@@ -2021,13 +2029,13 @@ do_ssl_start()
     SSL_set_fd(ssl_con,0);
 
     if (SSL_use_RSAPrivateKey(ssl_con,ssl_private_key)==0) {
-        sprintf(errstr,"ftpd: SSL_use_RSAPrivateKey %s",ERR_error_string(ERR_get_error()));
+        sprintf(errstr,"ftpd: SSL_use_RSAPrivateKey %s",ERR_error_string(ERR_get_error(),NULL));
 	perror_reply(421, errstr);
 	dologout(1);
     }
 
     if (SSL_use_certificate(ssl_con,ssl_public_cert)==0) {
-        sprintf(errstr,"ftpd: SSL_use_certificate %s",ERR_error_string(ERR_get_error()));
+        sprintf(errstr,"ftpd: SSL_use_certificate %s",ERR_error_string(ERR_get_error(),NULL));
 	perror_reply(421, errstr);
 	dologout(1);
     }
@@ -2036,7 +2044,7 @@ do_ssl_start()
 	    ssl_certsok_flag ? verify_callback : NULL);
 
     if (SSL_accept(ssl_con)<=0) {
-	sprintf(errstr,"ftpd: SSL_accept %s %s",ERR_error_string(ERR_get_error()));
+	sprintf(errstr,"ftpd: SSL_accept %s",ERR_error_string(ERR_get_error(),NULL));
 
 	perror_reply(421, errstr);
 	dologout(1);
